#ifndef _CSE451FS_H
#define _CSE451FS_H


#define CSE451_ROOT_INO 1	// warning: inode name 0 cannot be used 
				//	for an actual inode

/* Not the same as the bogus LINK_MAX in <linux/limits.h>. Oh well. */
#define CSE451_LINK_MAX	250

#define CSE451_SUPER_MAGIC	0x451F

#define CSE451_VALID_FS		0x0001		/* Clean fs. */
#define CSE451_ERROR_FS		0x0002		/* fs has errors. */

#define CSE451_INODES_PER_BLOCK ((BLOCK_SIZE)/CSE451_INODE_SIZE)

/*
 * The cse451 inode - bare bones.
 *   64 bytes / inode
 */
#define CSE451_NUMDATAPTRS 13
struct cse451_inode {
	__u16 i_mode;
	__u16 i_nlinks;
	__u16 i_uid;
	__u16 i_gid;
	__u32 i_filesize;
	__u32 i_datablocks[CSE451_NUMDATAPTRS];
};
#define CSE451_INODE_SIZE   (sizeof(struct cse451_inode))
#define CSE451_LINK_MAX	    65530

/*
 * cse451 super-block data on disk
 */
struct cse451_super_block {
	__u16 s_nNumInodes;		// inode map is tail of superblock
	__u16 s_nDataMapStart;		// block # of first data map block
	__u32 s_nDataMapBlocks;		// data map size, in blocks
	__u32 s_nInodeStart;		// block # of first inode block
	__u32 s_nNumInodeBlocks;	// number of blocks of inodes
	__u32 s_nDataBlocksStart;	// block # of first data block
	__u32 s_nDataBlocks;		// number of blocks of data

	__u32 s_nBusyInodes;		// number of inodes in use
	__u16 s_magic;			// magic number 
	char  s_imap[0];		// name for inode map
};
#define SUPERBLOCK_HEADERLENGTH (sizeof(struct cse451_super_block))

/*
 * the format of a directory entry
 */
#define CSE451_MAXDIRNAMELENGTH   30

struct cse451_dir_entry {
	__u16 inode;
	char name[CSE451_MAXDIRNAMELENGTH];
};

#define CSE451_DIRENTRYSIZE (sizeof(struct cse451_dir_entry))  // 32 bytes

/*
 * Note:  Ideally, we should have two .h files, one of interfaces
 *	we want to expose to standard user apps, and a separate
 *	one that includes what the kernel implementation needs
 *	(that #include's the user-directed .h, perhaps).
 *	But, for simplicity of management, we're going with just one.
 */

#ifdef __KERNEL__

/*
 * superblock_info - used by the kernel implementation of the cse451 fs
 */

struct cse451_sb_info {
	struct buffer_head*		sb_bh;
	struct cse451_super_block*	pSB;
	char*				pImap;
	struct buffer_head**		pDmap;
};

struct cse451_inode_info {
	__u32 i_datablocks[CSE451_NUMDATAPTRS];
};

#define CSE451_KERNEL_DEBUG

#ifdef CSE451_KERNEL_DEBUG
#define DEBUGPRINTK(args) printk args;
#else
#define DEBUGPRINTK(args)
#endif

#define DUMPX1(a) DEBUGPRINTK ((#a " = %02x\n", a))
#define DUMPX4(a) DEBUGPRINTK ((#a " = %08x\n", a))

// dir.c
extern struct dentry* cse451_lookup(struct inode*, struct dentry*);
extern int cse451_create(struct inode*, struct dentry*, int);
extern int cse451_mknod(struct inode*, struct dentry*, int, int);
extern int cse451_mkdir(struct inode * dir, struct dentry* dentry, int mode);
extern int cse451_rmdir(struct inode * dir, struct dentry* dentry);
extern int cse451_unlink(struct inode * dir, struct dentry* dentry);
extern int cse451_link(struct dentry*, struct inode*, struct dentry*);
extern int cse451_rename(struct inode*, struct dentry*,	struct inode*,
				struct dentry*);
extern int cse451_readdir(struct file*, void*, filldir_t);

// file.c
extern void cse451_truncate(struct inode *);
extern int cse451_sync_file(struct file*, struct dentry*, int);

// inode.c
extern struct inode* cse451_new_inode(const struct inode*, int*);
extern void cse451_delete_inode(struct inode*);
extern void cse451_free_inode(struct inode*);
extern void cse451_clear_inode(struct inode*);
extern void cse451_read_inode(struct inode*);
extern void cse451_write_inode(struct inode*, int);

// super.c
extern int cse451_new_block(struct inode* inode);
extern void cse451_free_block(struct inode*, int);
extern int get_block(struct inode *, long, struct buffer_head*, int);
extern struct buffer_head * cse451_getblk(struct inode*, int, int);
extern struct buffer_head * cse451_bread(struct inode*, int, int);
extern void cse451_write_super(struct super_block*);
extern void cse451_put_super(struct super_block*);
extern struct super_block *cse451_read_super(struct super_block *,void *,int);
extern int cse451_statfs(struct super_block *sb, struct statfs *buf);

// orphaned 
extern int cse451_bmap(struct inode *,int);
extern void cse451_put_inode(struct inode *);
extern int cse451_sync_inode(struct inode *);


// registration structures
extern struct inode_operations cse451_file_inode_operations;
extern struct inode_operations cse451_dir_inode_operations;
//extern struct inode_operations cse451_symlink_inode_operations;
extern struct file_operations cse451_dir_operations;
extern struct file_operations cse451_file_operations;
extern struct address_space_operations cse451_aops;

#endif /* __KERNEL__ */

#endif // _CSE451FS_H
